=== modified file 'NEWS'
--- NEWS	2008-04-09 07:36:41 +0000
+++ NEWS	2008-04-09 23:35:55 +0000
@@ -130,6 +130,11 @@
       non-ghost aware repository such as weaves will now fail if there are
       ghosts.  (Robert Collins)
 
+    * ``KnitVersionedFile`` no longer accepts an ``access_mode`` parameter, and
+      now requires the ``index`` and ``access_method`` parameters to be
+      supplied. A compatible shim has been kept in the new function
+      ``knit.make_file_knit``. (Robert Collins)
+
     * Log formatters must now provide log_revision instead of show and
       show_merge_revno methods. The latter had been deprecated since the 0.17
       release. (James Westby)
@@ -145,6 +150,27 @@
     * ``Transport.get_shared_medium`` is deprecated.  Use
       ``Transport.get_smart_medium`` instead.  (Andrew Bennetts)
 
+    * ``VersionedFile`` factories now accept a get_scope parameter rather
+      than using a call to ``transaction_finished``, allowing the removal of
+      the fixed list of versioned files per repository. (Robert Collins)
+
+    * ``VersionedFile.annotate_iter`` is deprecated. While in principle this
+      allowed lower memory use, all users of annotations wanted full file 
+      annotations, and there is no storage format suitable for incremental
+      line-by-line annotation. (Robert Collins)
+
+    * ``VersionedFile.clone_text`` is deprecated. This performance optimisation
+      is no longer used - reading the content of a file that is undergoing a
+      file level merge to identical state on two branches is rare enough, and
+      not expensive enough to special case. (Robert Collins)
+
+    * ``VersionedFile.clear_cache`` and ``enable_cache`` are deprecated.
+      These methods added significant complexity to the ``VersionedFile``
+      implementation, but were only used for optimising fetches from knits - 
+      which can be done from outside the knit layer, or via a caching
+      decorator. As knits are not the default format, the complexity is no
+      longer worth paying. (Robert Collins)
+
     * ``VersionedFile.create_empty`` is removed. This method presupposed a
       sensible mapping to a transport for individual files, but pack backed
       versioned files have no such mapping. (Robert Collins)
@@ -159,9 +185,23 @@
     * ``VersionedFile.get_parents`` is deprecated, please use
       ``VersionedFile.get_parent_map``. (Robert Collins)
 
+    * ``VersionedFile.get_sha1`` is deprecated, please use
+      ``VersionedFile.get_sha1s``. (Robert Collins)
+
     * ``VersionedFile.has_ghost`` is now deprecated, as it is both expensive
       and unused outside of a single test. (Robert Collins)
 
+    * ``VersionedFile.iter_parents`` is now deprecated in favour of
+      ``get_parent_map`` which can be used to instantiate a Graph on a
+      VersionedFile. (Robert Collins)
+
+    * ``VersionedFileStore`` no longer uses the transaction parameter given
+      to most methods; amongst other things this means that the
+      get_weave_or_empty method no longer guarantees errors on a missing weave
+      in a readonly transaction, and no longer caches versioned file instances
+      which reduces memory pressure (but requires more careful management by
+      callers to preserve performance). (Robert Collins)
+
   TESTING:
 
     * New -Dselftest_debug flag disables clearing of the debug flags during

=== modified file 'bzrlib/annotate.py'
--- bzrlib/annotate.py	2008-03-14 11:52:07 +0000
+++ bzrlib/annotate.py	2008-04-09 00:34:54 +0000
@@ -113,7 +113,7 @@
 def _annotations(repo, file_id, rev_id):
     """Return the list of (origin,text) for a revision of a file in a repository."""
     w = repo.weave_store.get_weave(file_id, repo.get_transaction())
-    return list(w.annotate_iter(rev_id))
+    return w.annotate(rev_id)
 
 
 def _annotate_file(branch, rev_id, file_id):

=== modified file 'bzrlib/fetch.py'
--- bzrlib/fetch.py	2008-03-19 04:39:04 +0000
+++ bzrlib/fetch.py	2008-04-09 20:13:46 +0000
@@ -27,8 +27,7 @@
 stored, so that if a revision is present we can totally recreate it.
 However, we can't know what files are included in a revision until we
 read its inventory.  So we query the inventory store of the source for
-the ids we need, and then pull those ids and finally actually join
-the inventories.
+the ids we need, and then pull those ids and then return to the inventories.
 """
 
 import bzrlib
@@ -168,13 +167,12 @@
                 if knit_kind == "file":
                     self._fetch_weave_text(file_id, revisions)
                 elif knit_kind == "inventory":
-                    # XXX:
-                    # Once we've processed all the files, then we generate the root
-                    # texts (if necessary), then we process the inventory.  It's a
-                    # bit distasteful to have knit_kind == "inventory" mean this,
-                    # perhaps it should happen on the first non-"file" knit, in case
-                    # it's not always inventory?
+                    # Before we process the inventory we generate the root
+                    # texts (if necessary) so that the inventories references
+                    # will be valid.
                     self._generate_root_texts(revs)
+                    # NB: This currently reopens the inventory weave in source;
+                    # using a full get_data_stream instead would avoid this.
                     self._fetch_inventory_weave(revs, pb)
                 elif knit_kind == "signatures":
                     # Nothing to do here; this will be taken care of when
@@ -217,15 +215,6 @@
         # we fetch all the texts, because texts do
         # not reference anything, and its cheap enough
         to_weave.join(from_weave, version_ids=required_versions)
-        # we don't need *all* of this data anymore, but we dont know
-        # what we do. This cache clearing will result in a new read 
-        # of the knit data when we do the checkout, but probably we
-        # want to emit the needed data on the fly rather than at the
-        # end anyhow.
-        # the from weave should know not to cache data being joined,
-        # but its ok to ask it to clear.
-        from_weave.clear_cache()
-        to_weave.clear_cache()
 
     def _fetch_inventory_weave(self, revs, pb):
         pb.update("fetch inventory", 0, 2)
@@ -243,7 +232,6 @@
             # corrupt.
             to_weave.join(from_weave, pb=child_pb, msg='merge inventory',
                           version_ids=revs)
-            from_weave.clear_cache()
         finally:
             child_pb.finished()
 

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2008-01-28 16:59:52 +0000
+++ bzrlib/inventory.py	2008-04-08 03:39:43 +0000
@@ -595,7 +595,7 @@
         # We can't check the length, because Weave doesn't store that
         # information, and the whole point of looking at the weave's
         # sha1sum is that we don't have to extract the text.
-        if self.text_sha1 != w.get_sha1(self.revision):
+        if self.text_sha1 != w.get_sha1s([self.revision])[0]:
             raise BzrCheckError('text {%s} version {%s} wrong sha1' 
                                 % (self.file_id, self.revision))
         checker.checked_texts[t] = self.text_sha1

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2008-04-07 05:59:51 +0000
+++ bzrlib/knit.py	2008-04-09 23:35:55 +0000
@@ -144,10 +144,6 @@
     def __init__(self):
         self._should_strip_eol = False
 
-    def annotate(self):
-        """Return a list of (origin, text) tuples."""
-        return list(self.annotate_iter())
-
     def apply_delta(self, delta, new_version_id):
         """Apply delta to this object to become new_version_id."""
         raise NotImplementedError(self.apply_delta)
@@ -206,9 +202,9 @@
         KnitContent.__init__(self)
         self._lines = lines
 
-    def annotate_iter(self):
-        """Yield tuples of (origin, text) for each content line."""
-        return iter(self._lines)
+    def annotate(self):
+        """Return a list of (origin, text) for each content line."""
+        return list(self._lines)
 
     def apply_delta(self, delta, new_version_id):
         """Apply delta to this object to become new_version_id."""
@@ -256,10 +252,9 @@
         self._lines = lines
         self._version_id = version_id
 
-    def annotate_iter(self):
-        """Yield tuples of (origin, text) for each content line."""
-        for line in self._lines:
-            yield self._version_id, line
+    def annotate(self):
+        """Return a list of (origin, text) for each content line."""
+        return [(self._version_id, line) for line in self._lines]
 
     def apply_delta(self, delta, new_version_id):
         """Apply delta to this object to become new_version_id."""
@@ -427,9 +422,9 @@
                        for origin, text in lines)
         return out
 
-    def annotate_iter(self, knit, version_id):
+    def annotate(self, knit, version_id):
         content = knit._get_content(version_id)
-        return content.annotate_iter()
+        return content.annotate()
 
 
 class KnitPlainFactory(_KnitFactory):
@@ -489,14 +484,42 @@
             out.extend(lines)
         return out
 
-    def annotate_iter(self, knit, version_id):
+    def annotate(self, knit, version_id):
         annotator = _KnitAnnotator(knit)
-        return iter(annotator.annotate(version_id))
+        return annotator.annotate(version_id)
 
 
 def make_empty_knit(transport, relpath):
     """Construct a empty knit at the specified location."""
-    k = KnitVersionedFile(transport, relpath, 'w', KnitPlainFactory)
+    k = make_file_knit(transport, relpath, 'w', KnitPlainFactory)
+
+
+def make_file_knit(name, transport, file_mode=None, access_mode='w',
+    factory=None, delta=True, create=False, create_parent_dir=False,
+    delay_create=False, dir_mode=None, get_scope=None):
+    """Factory to create a KnitVersionedFile for a .knit/.kndx file pair."""
+    if factory is None:
+        factory = KnitAnnotateFactory()
+    else:
+        factory = KnitPlainFactory()
+    if get_scope is None:
+        get_scope = lambda:None
+    index = _KnitIndex(transport, name + INDEX_SUFFIX,
+        access_mode, create=create, file_mode=file_mode,
+        create_parent_dir=create_parent_dir, delay_create=delay_create,
+        dir_mode=dir_mode, get_scope=get_scope)
+    access = _KnitAccess(transport, name + DATA_SUFFIX, file_mode,
+        dir_mode, ((create and not len(index)) and delay_create),
+        create_parent_dir)
+    return KnitVersionedFile(name, transport, factory=factory,
+        create=create, delay_create=delay_create, index=index,
+        access_method=access)
+
+
+def get_suffixes():
+    """Return the suffixes used by file based knits."""
+    return [DATA_SUFFIX, INDEX_SUFFIX]
+make_file_knit.get_suffixes = get_suffixes
 
 
 class KnitVersionedFile(VersionedFile):
@@ -514,7 +537,7 @@
     stored and retrieved.
     """
 
-    def __init__(self, relpath, transport, file_mode=None, access_mode=None,
+    def __init__(self, relpath, transport, file_mode=None,
         factory=None, delta=True, create=False, create_parent_dir=False,
         delay_create=False, dir_mode=None, index=None, access_method=None):
         """Construct a knit at location specified by relpath.
@@ -527,30 +550,18 @@
             actually be created until the first data is stored.
         :param index: An index to use for the knit.
         """
-        if access_mode is None:
-            access_mode = 'w'
-        super(KnitVersionedFile, self).__init__(access_mode)
-        assert access_mode in ('r', 'w'), "invalid mode specified %r" % access_mode
+        super(KnitVersionedFile, self).__init__()
         self.transport = transport
         self.filename = relpath
         self.factory = factory or KnitAnnotateFactory()
-        self.writable = (access_mode == 'w')
         self.delta = delta
 
         self._max_delta_chain = 200
 
-        if index is None:
-            self._index = _KnitIndex(transport, relpath + INDEX_SUFFIX,
-                access_mode, create=create, file_mode=file_mode,
-                create_parent_dir=create_parent_dir, delay_create=delay_create,
-                dir_mode=dir_mode)
-        else:
-            self._index = index
-        if access_method is None:
-            _access = _KnitAccess(transport, relpath + DATA_SUFFIX, file_mode, dir_mode,
-                ((create and not len(self)) and delay_create), create_parent_dir)
-        else:
-            _access = access_method
+        if None in (access_method, index):
+            raise ValueError("No default access_method or index any more")
+        self._index = index
+        _access = access_method
         if create and not len(self) and not delay_create:
             _access.create()
         self._data = _KnitData(_access)
@@ -588,6 +599,9 @@
 
         return fulltext_size > delta_size
 
+    def _check_write_ok(self):
+        return self._index._check_write_ok()
+
     def _add_raw_records(self, records, data):
         """Add all the records 'records' with data pre-joined in 'data'.
 
@@ -604,19 +618,9 @@
         for (version_id, options, parents, size), access_memo in zip(
             records, positions):
             index_entries.append((version_id, options, access_memo, parents))
-            if self._data._do_cache:
-                self._data._cache[version_id] = data[offset:offset+size]
             offset += size
         self._index.add_versions(index_entries)
 
-    def enable_cache(self):
-        """Start caching data for this knit"""
-        self._data.enable_cache()
-
-    def clear_cache(self):
-        """Clear the data cache only."""
-        self._data.clear_cache()
-
     def copy_to(self, name, transport):
         """See VersionedFile.copy_to()."""
         # copy the current index to a temp index to avoid racing with local
@@ -758,20 +762,12 @@
         """See VersionedFile.get_graph_with_ghosts()."""
         return self.get_parent_map(self.versions())
 
-    def get_sha1(self, version_id):
-        return self.get_sha1s([version_id])[0]
-
     def get_sha1s(self, version_ids):
-        """See VersionedFile.get_sha1()."""
+        """See VersionedFile.get_sha1s()."""
         record_map = self._get_record_map(version_ids)
         # record entry 2 is the 'digest'.
         return [record_map[v][2] for v in version_ids]
 
-    @staticmethod
-    def get_suffixes():
-        """See VersionedFile.get_suffixes()."""
-        return [DATA_SUFFIX, INDEX_SUFFIX]
-
     @deprecated_method(one_four)
     def has_ghost(self, version_id):
         """True if there is a ghost reference in the file to version_id."""
@@ -820,7 +816,7 @@
                 # Also check the SHA-1 of the fulltext this content will
                 # produce.
                 raw_data = reader_callable(length)
-                my_fulltext_sha1 = self.get_sha1(version_id)
+                my_fulltext_sha1 = self.get_sha1s([version_id])[0]
                 df, rec = self._data._parse_record_header(version_id, raw_data)
                 stream_fulltext_sha1 = rec[3]
                 if my_fulltext_sha1 != stream_fulltext_sha1:
@@ -1090,12 +1086,6 @@
     def check(self, progress_bar=None):
         """See VersionedFile.check()."""
 
-    def _clone_text(self, new_version_id, old_version_id, parents):
-        """See VersionedFile.clone_text()."""
-        # FIXME RBC 20060228 make fast by only inserting an index with null 
-        # delta.
-        self.add_lines(new_version_id, parents, self.get_lines(old_version_id))
-
     def get_lines(self, version_id):
         """See VersionedFile.get_lines()."""
         return self.get_line_list([version_id])[0]
@@ -1239,26 +1229,15 @@
 
         pb.update('Walking content.', total, total)
         
-    def iter_parents(self, version_ids):
-        """Iterate through the parents for many version ids.
-
-        :param version_ids: An iterable yielding version_ids.
-        :return: An iterator that yields (version_id, parents). Requested 
-            version_ids not present in the versioned file are simply skipped.
-            The order is undefined, allowing for different optimisations in
-            the underlying implementation.
-        """
-        return self._index.iter_parents(version_ids)
-
     def num_versions(self):
         """See VersionedFile.num_versions()."""
         return self._index.num_versions()
 
     __len__ = num_versions
 
-    def annotate_iter(self, version_id):
-        """See VersionedFile.annotate_iter."""
-        return self.factory.annotate_iter(self, version_id)
+    def annotate(self, version_id):
+        """See VersionedFile.annotate."""
+        return self.factory.annotate(self, version_id)
 
     def get_parent_map(self, version_ids):
         """See VersionedFile.get_parent_map."""
@@ -1400,8 +1379,15 @@
                                    parents,
                                    index)
 
+    def _check_write_ok(self):
+        if self._get_scope() != self._scope:
+            raise errors.OutSideTransaction()
+        if self._mode != 'w':
+            raise errors.ReadOnlyObjectDirtiedError(self)
+
     def __init__(self, transport, filename, mode, create=False, file_mode=None,
-                 create_parent_dir=False, delay_create=False, dir_mode=None):
+        create_parent_dir=False, delay_create=False, dir_mode=None,
+        get_scope=None):
         _KnitComponentFile.__init__(self, transport, filename, mode,
                                     file_mode=file_mode,
                                     create_parent_dir=create_parent_dir,
@@ -1428,6 +1414,8 @@
             else:
                 self._transport.put_bytes_non_atomic(
                     self._filename, self.HEADER, mode=self._file_mode)
+        self._scope = get_scope()
+        self._get_scope = get_scope
 
     def get_ancestry(self, versions, topo_sorted=True):
         """See VersionedFile.get_ancestry."""
@@ -1505,28 +1493,6 @@
                                   parents, (method, noeol))
         return result
 
-    def iter_parents(self, version_ids):
-        """Iterate through the parents for many version ids.
-
-        :param version_ids: An iterable yielding version_ids.
-        :return: An iterator that yields (version_id, parents). Requested 
-            version_ids not present in the versioned file are simply skipped.
-            The order is undefined, allowing for different optimisations in
-            the underlying implementation.
-        """
-        parent_map = self.get_parent_map(version_ids)
-        parent_map_set = set(parent_map)
-        unknown_existence = set()
-        for parents in parent_map.itervalues():
-            unknown_existence.update(parents)
-        unknown_existence.difference_update(parent_map_set)
-        present_parents = set(self.get_parent_map(unknown_existence))
-        present_parents.update(parent_map_set)
-        for version_id, parents in parent_map.iteritems():
-            parents = tuple(parent for parent in parents
-                if parent in present_parents)
-            yield version_id, parents
-
     def num_versions(self):
         return len(self._history)
 
@@ -1676,6 +1642,9 @@
             raise KnitCorrupt(self, "Cannot do delta compression without "
                 "parent tracking.")
 
+    def _check_write_ok(self):
+        pass
+
     def _get_entries(self, keys, check_present=False):
         """Get the entries for keys.
         
@@ -1846,35 +1815,6 @@
         else:
             return 'fulltext'
 
-    def iter_parents(self, version_ids):
-        """Iterate through the parents for many version ids.
-
-        :param version_ids: An iterable yielding version_ids.
-        :return: An iterator that yields (version_id, parents). Requested 
-            version_ids not present in the versioned file are simply skipped.
-            The order is undefined, allowing for different optimisations in
-            the underlying implementation.
-        """
-        if self._parents:
-            all_nodes = set(self._get_entries(self._version_ids_to_keys(version_ids)))
-            all_parents = set()
-            present_parents = set()
-            for node in all_nodes:
-                all_parents.update(node[3][0])
-                # any node we are querying must be present
-                present_parents.add(node[1])
-            unknown_parents = all_parents.difference(present_parents)
-            present_parents.update(self._present_keys(unknown_parents))
-            for node in all_nodes:
-                parents = []
-                for parent in node[3][0]:
-                    if parent in present_parents:
-                        parents.append(parent[0])
-                yield node[1][0], tuple(parents)
-        else:
-            for node in self._get_entries(self._version_ids_to_keys(version_ids)):
-                yield node[1][0], ()
-
     def num_versions(self):
         return len(list(self._graph_index.iter_all_entries()))
 
@@ -2421,23 +2361,6 @@
         """Get all the versions in the stream."""
         return self._by_version.keys()
 
-    def iter_parents(self, version_ids):
-        """Iterate through the parents for many version ids.
-
-        :param version_ids: An iterable yielding version_ids.
-        :return: An iterator that yields (version_id, parents). Requested 
-            version_ids not present in the versioned file are simply skipped.
-            The order is undefined, allowing for different optimisations in
-            the underlying implementation.
-        """
-        result = []
-        for version in version_ids:
-            try:
-                result.append((version, self._by_version[version][2]))
-            except KeyError:
-                pass
-        return result
-
 
 class _KnitData(object):
     """Manage extraction of data from a KnitAccess, caching and decompressing.
@@ -2455,21 +2378,6 @@
         """
         self._access = access
         self._checked = False
-        # TODO: jam 20060713 conceptually, this could spill to disk
-        #       if the cached size gets larger than a certain amount
-        #       but it complicates the model a bit, so for now just use
-        #       a simple dictionary
-        self._cache = {}
-        self._do_cache = False
-
-    def enable_cache(self):
-        """Enable caching of reads."""
-        self._do_cache = True
-
-    def clear_cache(self):
-        """Clear the record cache."""
-        self._do_cache = False
-        self._cache = {}
 
     def _open_file(self):
         return self._access.open_file()
@@ -2575,29 +2483,15 @@
         # uses readv so nice and fast we hope.
         if len(records):
             # grab the disk data needed.
-            if self._cache:
-                # Don't check _cache if it is empty
-                needed_offsets = [index_memo for version_id, index_memo
-                                              in records
-                                              if version_id not in self._cache]
-            else:
-                needed_offsets = [index_memo for version_id, index_memo
-                                               in records]
-
+            needed_offsets = [index_memo for version_id, index_memo
+                                           in records]
             raw_records = self._access.get_raw_records(needed_offsets)
 
         for version_id, index_memo in records:
-            if version_id in self._cache:
-                # This data has already been validated
-                data = self._cache[version_id]
-            else:
-                data = raw_records.next()
-                if self._do_cache:
-                    self._cache[version_id] = data
-
-                # validate the header
-                df, rec = self._parse_record_header(version_id, data)
-                df.close()
+            data = raw_records.next()
+            # validate the header
+            df, rec = self._parse_record_header(version_id, data)
+            df.close()
             yield version_id, data
 
     def read_records_iter(self, records):
@@ -2613,24 +2507,7 @@
         if not records:
             return
 
-        if self._cache:
-            # Skip records we have alread seen
-            yielded_records = set()
-            needed_records = set()
-            for record in records:
-                if record[0] in self._cache:
-                    if record[0] in yielded_records:
-                        continue
-                    yielded_records.add(record[0])
-                    data = self._cache[record[0]]
-                    content, digest = self._parse_record(record[0], data)
-                    yield (record[0], content, digest)
-                else:
-                    needed_records.add(record)
-            needed_records = sorted(needed_records, key=operator.itemgetter(1))
-        else:
-            needed_records = sorted(set(records), key=operator.itemgetter(1))
-
+        needed_records = sorted(set(records), key=operator.itemgetter(1))
         if not needed_records:
             return
 
@@ -2642,8 +2519,6 @@
         for (version_id, index_memo), data in \
                 izip(iter(needed_records), raw_data):
             content, digest = self._parse_record(version_id, data)
-            if self._do_cache:
-                self._cache[version_id] = data
             yield version_id, content, digest
 
     def read_records(self, records):
@@ -2658,8 +2533,8 @@
 class InterKnit(InterVersionedFile):
     """Optimised code paths for knit to knit operations."""
     
-    _matching_file_from_factory = KnitVersionedFile
-    _matching_file_to_factory = KnitVersionedFile
+    _matching_file_from_factory = staticmethod(make_file_knit)
+    _matching_file_to_factory = staticmethod(make_file_knit)
     
     @staticmethod
     def is_compatible(source, target):
@@ -2822,7 +2697,7 @@
     """Optimised code paths for weave to knit operations."""
     
     _matching_file_from_factory = bzrlib.weave.WeaveFile
-    _matching_file_to_factory = KnitVersionedFile
+    _matching_file_to_factory = staticmethod(make_file_knit)
     
     @staticmethod
     def is_compatible(source, target):

=== modified file 'bzrlib/multiparent.py'
--- bzrlib/multiparent.py	2008-03-19 04:39:04 +0000
+++ bzrlib/multiparent.py	2008-04-09 00:34:54 +0000
@@ -382,7 +382,7 @@
                     if [p for p in parents if p not in self._parents] != []:
                         continue
                     lines = [a + ' ' + l for a, l in
-                             vf.annotate_iter(revision)]
+                             vf.annotate(revision)]
                     if snapshots is None:
                         force_snapshot = None
                     else:

=== modified file 'bzrlib/repofmt/knitrepo.py'
--- bzrlib/repofmt/knitrepo.py	2008-03-26 21:42:35 +0000
+++ bzrlib/repofmt/knitrepo.py	2008-04-04 00:06:58 +0000
@@ -101,6 +101,9 @@
         self._commit_builder_class = _commit_builder_class
         self._serializer = _serializer
         self._reconcile_fixes_text_parents = True
+        control_store.get_scope = self.get_transaction
+        text_store.get_scope = self.get_transaction
+        _revision_store.get_scope = self.get_transaction
 
     def _warn_if_deprecated(self):
         # This class isn't deprecated
@@ -358,7 +361,7 @@
             repo_transport,
             prefixed=False,
             file_mode=control_files._file_mode,
-            versionedfile_class=knit.KnitVersionedFile,
+            versionedfile_class=knit.make_file_knit,
             versionedfile_kwargs={'factory':knit.KnitPlainFactory()},
             )
 
@@ -369,7 +372,7 @@
             file_mode=control_files._file_mode,
             prefixed=False,
             precious=True,
-            versionedfile_class=knit.KnitVersionedFile,
+            versionedfile_class=knit.make_file_knit,
             versionedfile_kwargs={'delta':False,
                                   'factory':knit.KnitPlainFactory(),
                                  },
@@ -382,7 +385,7 @@
         return self._get_versioned_file_store('knits',
                                   transport,
                                   control_files,
-                                  versionedfile_class=knit.KnitVersionedFile,
+                                  versionedfile_class=knit.make_file_knit,
                                   versionedfile_kwargs={
                                       'create_parent_dir':True,
                                       'delay_create':True,

=== modified file 'bzrlib/repofmt/pack_repo.py'
--- bzrlib/repofmt/pack_repo.py	2008-03-26 21:42:35 +0000
+++ bzrlib/repofmt/pack_repo.py	2008-04-04 00:06:58 +0000
@@ -1061,7 +1061,6 @@
             deltas=True, parents=True)
         output_knit = knit.KnitVersionedFile('reconcile-texts',
             self._pack_collection.transport,
-            None,
             index=knit_index,
             access_method=_PackAccess(
                 {self.new_pack.text_index:self.new_pack.access_tuple()},
@@ -1706,7 +1705,7 @@
         self.repo._revision_knit = knit.KnitVersionedFile(
             'revisions', self.transport.clone('..'),
             self.repo.control_files._file_mode,
-            create=False, access_mode=self.repo._access_mode(),
+            create=False,
             index=knit_index, delta=False, factory=knit.KnitPlainFactory(),
             access_method=self.repo._pack_collection.revision_index.knit_access)
         return self.repo._revision_knit
@@ -1724,7 +1723,7 @@
         self.repo._signature_knit = knit.KnitVersionedFile(
             'signatures', self.transport.clone('..'),
             self.repo.control_files._file_mode,
-            create=False, access_mode=self.repo._access_mode(),
+            create=False,
             index=knit_index, delta=False, factory=knit.KnitPlainFactory(),
             access_method=self.repo._pack_collection.signature_index.knit_access)
         return self.repo._signature_knit
@@ -1813,7 +1812,7 @@
         return knit.KnitVersionedFile(
             'inventory', self.transport.clone('..'),
             self.repo.control_files._file_mode,
-            create=False, access_mode=self.repo._access_mode(),
+            create=False,
             index=knit_index, delta=True, factory=knit.KnitPlainFactory(),
             access_method=self.repo._pack_collection.inventory_index.knit_access)
 
@@ -1849,15 +1848,6 @@
     def _abort_write_group(self):
         self._pack_collection._abort_write_group()
 
-    def _access_mode(self):
-        """Return 'w' or 'r' for depending on whether a write lock is active.
-        
-        This method is a helper for the Knit-thunking support objects.
-        """
-        if self.is_write_locked():
-            return 'w'
-        return 'r'
-
     def _find_inconsistent_revision_parents(self):
         """Find revisions with incorrectly cached parents.
 
@@ -2121,7 +2111,7 @@
             repo_transport,
             prefixed=False,
             file_mode=control_files._file_mode,
-            versionedfile_class=knit.KnitVersionedFile,
+            versionedfile_class=knit.make_file_knit,
             versionedfile_kwargs={'factory': knit.KnitPlainFactory()},
             )
 
@@ -2132,7 +2122,7 @@
             file_mode=control_files._file_mode,
             prefixed=False,
             precious=True,
-            versionedfile_class=knit.KnitVersionedFile,
+            versionedfile_class=knit.make_file_knit,
             versionedfile_kwargs={'delta': False,
                                   'factory': knit.KnitPlainFactory(),
                                  },
@@ -2145,7 +2135,7 @@
         return self._get_versioned_file_store('knits',
                                   transport,
                                   control_files,
-                                  versionedfile_class=knit.KnitVersionedFile,
+                                  versionedfile_class=knit.make_file_knit,
                                   versionedfile_kwargs={
                                       'create_parent_dir': True,
                                       'delay_create': True,

=== modified file 'bzrlib/repofmt/weaverepo.py'
--- bzrlib/repofmt/weaverepo.py	2008-03-26 21:42:35 +0000
+++ bzrlib/repofmt/weaverepo.py	2008-04-08 21:41:15 +0000
@@ -37,7 +37,7 @@
 from bzrlib.decorators import needs_read_lock, needs_write_lock
 from bzrlib.repository import (
     CommitBuilder,
-    MetaDirRepository,
+    MetaDirVersionedFileRepository,
     MetaDirRepositoryFormat,
     Repository,
     RepositoryFormat,
@@ -77,6 +77,9 @@
             self.inventory_store = get_store('inventory-store')
             text_store = get_store('text-store')
         super(AllInOneRepository, self).__init__(_format, a_bzrdir, a_bzrdir._control_files, _revision_store, control_store, text_store)
+        if control_store is not None:
+            control_store.get_scope = self.get_transaction
+        text_store.get_scope = self.get_transaction
 
     @needs_read_lock
     def _all_possible_ids(self):
@@ -214,7 +217,7 @@
         return False
 
 
-class WeaveMetaDirRepository(MetaDirRepository):
+class WeaveMetaDirRepository(MetaDirVersionedFileRepository):
     """A subclass of MetaDirRepository to set weave specific policy."""
 
     _serializer = xml5.serializer_v5
@@ -629,7 +632,6 @@
         result = versionedfile.add_lines(
             self._new_revision_id, parents, new_lines,
             nostore_sha=nostore_sha)[0:2]
-        versionedfile.clear_cache()
         return result
 
 

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2008-03-28 03:54:40 +0000
+++ bzrlib/repository.py	2008-04-08 22:47:55 +0000
@@ -420,13 +420,10 @@
         # implementation could give us bad output from readlines() so this is
         # not a guarantee of safety. What would be better is always checking
         # the content during test suite execution. RBC 20070912
-        try:
-            return versionedfile.add_lines_with_ghosts(
-                self._new_revision_id, parents, new_lines,
-                nostore_sha=nostore_sha, random_id=self.random_revid,
-                check_content=False)[0:2]
-        finally:
-            versionedfile.clear_cache()
+        return versionedfile.add_lines_with_ghosts(
+            self._new_revision_id, parents, new_lines,
+            nostore_sha=nostore_sha, random_id=self.random_revid,
+            check_content=False)[0:2]
 
 
 class RootCommitBuilder(CommitBuilder):
@@ -1281,16 +1278,18 @@
                 setdefault(file_id, set()).add(revision_id)
         return result
 
-    def fileids_altered_by_revision_ids(self, revision_ids):
+    def fileids_altered_by_revision_ids(self, revision_ids, _inv_weave=None):
         """Find the file ids and versions affected by revisions.
 
         :param revisions: an iterable containing revision ids.
+        :param _inv_weave: The inventory weave from this repository or None.
+            If None, the inventory weave will be opened automatically.
         :return: a dictionary mapping altered file-ids to an iterable of
         revision_ids. Each altered file-ids has the exact revision_ids that
         altered it listed explicitly.
         """
         selected_revision_ids = set(revision_ids)
-        w = self.get_inventory_weave()
+        w = _inv_weave or self.get_inventory_weave()
         pb = ui.ui_factory.nested_progress_bar()
         try:
             return self._find_file_ids_from_xml_inventory_lines(
@@ -1450,10 +1449,9 @@
         # processed?
         self.lock_read()
         inv_w = self.get_inventory_weave()
-        inv_w.enable_cache()
 
         # file ids that changed
-        file_ids = self.fileids_altered_by_revision_ids(revision_ids)
+        file_ids = self.fileids_altered_by_revision_ids(revision_ids, inv_w)
         count = 0
         num_file_ids = len(file_ids)
         for file_id, altered_versions in file_ids.iteritems():
@@ -1467,7 +1465,6 @@
 
         # inventory
         yield ("inventory", None, revision_ids)
-        inv_w.clear_cache()
 
         # signatures
         revisions_with_signatures = set()
@@ -2038,6 +2035,17 @@
         return not self.control_files._transport.has('no-working-trees')
 
 
+class MetaDirVersionedFileRepository(MetaDirRepository):
+    """Repositories in a meta-dir, that work via versioned file objects."""
+
+    def __init__(self, _format, a_bzrdir, control_files, _revision_store, control_store, text_store):
+        super(MetaDirVersionedFileRepository, self).__init__(_format, a_bzrdir,
+            control_files, _revision_store, control_store, text_store)
+        _revision_store.get_scope = self.get_transaction
+        control_store.get_scope = self.get_transaction
+        text_store.get_scope = self.get_transaction
+
+
 class RepositoryFormatRegistry(registry.Registry):
     """Registry of RepositoryFormats."""
 

=== modified file 'bzrlib/revisiontree.py'
--- bzrlib/revisiontree.py	2007-12-06 20:06:01 +0000
+++ bzrlib/revisiontree.py	2008-04-09 00:34:54 +0000
@@ -93,7 +93,7 @@
                       default_revision=revision.CURRENT_REVISION):
         """See Tree.annotate_iter"""
         w = self._get_weave(file_id)
-        return w.annotate_iter(self.inventory[file_id].revision)
+        return w.annotate(self.inventory[file_id].revision)
 
     def get_file_size(self, file_id):
         return self._inventory[file_id].text_size

=== modified file 'bzrlib/store/revision/knit.py'
--- bzrlib/store/revision/knit.py	2007-10-11 04:54:04 +0000
+++ bzrlib/store/revision/knit.py	2008-04-04 00:06:58 +0000
@@ -26,7 +26,7 @@
     osutils,
     revision as _mod_revision,
     )
-from bzrlib.knit import KnitVersionedFile, KnitPlainFactory
+from bzrlib.knit import make_file_knit, KnitPlainFactory
 from bzrlib.store.revision import RevisionStore
 from bzrlib.store.versioned import VersionedFileStore
 from bzrlib.transport import get_transport
@@ -42,7 +42,7 @@
         versioned_file_store = VersionedFileStore(
             t.clone('revision-store'),
             precious=True,
-            versionedfile_class=KnitVersionedFile,
+            versionedfile_class=make_file_knit,
             versionedfile_kwargs={'delta':False, 'factory':KnitPlainFactory()})
         return KnitRevisionStore(versioned_file_store)
 

=== modified file 'bzrlib/store/versioned/__init__.py'
--- bzrlib/store/versioned/__init__.py	2007-09-25 08:14:12 +0000
+++ bzrlib/store/versioned/__init__.py	2008-04-08 00:57:07 +0000
@@ -54,21 +54,8 @@
         self._precious = precious
         self._versionedfile_class = versionedfile_class
         self._versionedfile_kwargs = versionedfile_kwargs
-
-    def _clear_cache_id(self, file_id, transaction):
-        """WARNING may lead to inconsistent object references for file_id.
-
-        Remove file_id from the transaction map. 
-
-        NOT in the transaction api because theres no reliable way to clear
-        callers. So its here for very specialised use rather than having an
-        'api' that isn't.
-        """
-        weave = transaction.map.find_weave(file_id)
-        if weave is not None:
-            mutter("old data in transaction in %s for %s", self, file_id)
-            # FIXME abstraction violation - transaction now has stale data.
-            transaction.map.remove_object(weave)
+        # Used for passing get_scope to versioned file constructors;
+        self.get_scope = None
 
     def filename(self, file_id):
         """Return the path relative to the transport root."""
@@ -108,7 +95,6 @@
         filename = self.filename(file_id)
         for suffix in suffixes:
             self._transport.delete(filename + suffix)
-        self._clear_cache_id(file_id, transaction)
 
     def _get(self, file_id):
         return self._transport.get(self.filename(file_id))
@@ -130,26 +116,19 @@
         file_id. This is used to reduce duplicate filename calculations when
         using 'get_weave_or_empty'. FOR INTERNAL USE ONLY.
         """
-        weave = transaction.map.find_weave(file_id)
-        if weave is not None:
-            #mutter("cache hit in %s for %s", self, file_id)
-            return weave
         if _filename is None:
             _filename = self.filename(file_id)
         if transaction.writeable():
             w = self._versionedfile_class(_filename, self._transport, self._file_mode,
-                                          **self._versionedfile_kwargs)
-            transaction.map.add_weave(file_id, w)
-            transaction.register_dirty(w)
+                get_scope=self.get_scope, **self._versionedfile_kwargs)
         else:
             w = self._versionedfile_class(_filename,
                                           self._transport,
                                           self._file_mode,
                                           create=False,
                                           access_mode='r',
+                                          get_scope=self.get_scope,
                                           **self._versionedfile_kwargs)
-            transaction.map.add_weave(file_id, w)
-            transaction.register_clean(w, precious=self._precious)
         return w
 
     def _make_new_versionedfile(self, file_id, transaction,
@@ -168,15 +147,16 @@
             # we try without making the directory first because thats optimising
             # for the common case.
             weave = self._versionedfile_class(_filename, self._transport, self._file_mode, create=True,
-                                              **self._versionedfile_kwargs)
+                get_scope=self.get_scope, **self._versionedfile_kwargs)
         except errors.NoSuchFile:
             if not self._prefixed:
                 # unexpected error - NoSuchFile is expected to be raised on a
                 # missing dir only and that only occurs when we are prefixed.
                 raise
             self._transport.mkdir(self.hash_prefix(file_id), mode=self._dir_mode)
-            weave = self._versionedfile_class(_filename, self._transport, 
+            weave = self._versionedfile_class(_filename, self._transport,
                                               self._file_mode, create=True,
+                                              get_scope=self.get_scope,
                                               **self._versionedfile_kwargs)
         return weave
 
@@ -193,9 +173,6 @@
         except errors.NoSuchFile:
             weave = self._make_new_versionedfile(file_id, transaction,
                 known_missing=True, _filename=_filename)
-            transaction.map.add_weave(file_id, weave)
-            # has to be dirty - its able to mutate on its own.
-            transaction.register_dirty(weave)
             return weave
 
     def _put_weave(self, file_id, weave, transaction):
@@ -205,7 +182,6 @@
 
     def copy(self, source, result_id, transaction):
         """Copy the source versioned file to result_id in this store."""
-        self._clear_cache_id(result_id, transaction)
         source.copy_to(self.filename(result_id), self._transport)
  
     def copy_all_ids(self, store_from, pb=None, from_transaction=None,

=== modified file 'bzrlib/tests/interversionedfile_implementations/__init__.py'
--- bzrlib/tests/interversionedfile_implementations/__init__.py	2007-06-28 06:16:19 +0000
+++ bzrlib/tests/interversionedfile_implementations/__init__.py	2008-04-04 00:06:58 +0000
@@ -73,11 +73,11 @@
         """Generate the default list of interversionedfile permutations to test."""
         from bzrlib.versionedfile import InterVersionedFile
         from bzrlib.weave import WeaveFile
-        from bzrlib.knit import KnitVersionedFile
+        from bzrlib.knit import make_file_knit
         result = []
         # test the fallback InterVersionedFile from annotated knits to weave
         result.append((InterVersionedFile,
-                       KnitVersionedFile,
+                       make_file_knit,
                        WeaveFile))
         for optimiser in InterVersionedFile._optimisers:
             result.append((optimiser,

=== modified file 'bzrlib/tests/repository_implementations/test_check_reconcile.py'
--- bzrlib/tests/repository_implementations/test_check_reconcile.py	2008-03-19 04:39:04 +0000
+++ bzrlib/tests/repository_implementations/test_check_reconcile.py	2008-04-08 03:39:43 +0000
@@ -153,7 +153,7 @@
         :returns: A dict of `{version: hash}`.
         """
         vf = repo.weave_store.get_weave('a-file-id', repo.get_transaction())
-        return dict((v, vf.get_sha1(v)) for v in versions)
+        return dict(zip(versions, vf.get_sha1s(versions)))
 
     def test_reconcile_behaviour(self):
         """Populate a repository and reconcile it, verifying the state before

=== modified file 'bzrlib/tests/revisionstore_implementations/test_all.py'
--- bzrlib/tests/revisionstore_implementations/test_all.py	2007-08-23 02:18:37 +0000
+++ bzrlib/tests/revisionstore_implementations/test_all.py	2008-04-04 00:06:58 +0000
@@ -37,6 +37,10 @@
         super(TestAll, self).setUp()
         self.store = self.store_factory.create(self.get_url('.'))
         self.transaction = PassThroughTransaction()
+        self.store.get_scope = self.get_transaction
+
+    def get_transaction(self):
+        return self.transaction
 
     def test_add_has_get(self):
         rev = self.add_sample_rev()

=== modified file 'bzrlib/tests/test_fetch.py'
--- bzrlib/tests/test_fetch.py	2008-03-26 21:44:02 +0000
+++ bzrlib/tests/test_fetch.py	2008-04-08 00:57:07 +0000
@@ -285,9 +285,13 @@
         # unfortunately this log entry is branch format specific. We could 
         # factor out the 'what files does this format use' to a method on the 
         # repository, which would let us to this generically. RBC 20060419
+        # RBC 20080408: Or perhaps we can assert that no files are fully read
+        # twice?
         self.assertEqual(1, self._count_log_matches('/ce/id.kndx', http_logs))
         self.assertEqual(1, self._count_log_matches('/ce/id.knit', http_logs))
-        self.assertEqual(1, self._count_log_matches('inventory.kndx', http_logs))
+        # XXX: This *should* be '1', but more intrusive fetch changes are
+        # needed to drop this back to 1.
+        self.assertEqual(2, self._count_log_matches('inventory.kndx', http_logs))
         # this r-h check test will prevent regressions, but it currently already 
         # passes, before the patch to cache-rh is applied :[
         self.assertTrue(1 >= self._count_log_matches('revision-history',
@@ -311,7 +315,8 @@
         self.log('\n'.join(http_logs))
         self.assertEqual(1, self._count_log_matches('branch-format', http_logs))
         self.assertEqual(1, self._count_log_matches('branch/format', http_logs))
-        self.assertEqual(1, self._count_log_matches('repository/format', http_logs))
+        self.assertEqual(1, self._count_log_matches('repository/format',
+            http_logs))
         self.assertTrue(1 >= self._count_log_matches('revision-history',
                                                      http_logs))
         self.assertTrue(1 >= self._count_log_matches('last-revision',

=== modified file 'bzrlib/tests/test_knit.py'
--- bzrlib/tests/test_knit.py	2008-04-07 05:59:51 +0000
+++ bzrlib/tests/test_knit.py	2008-04-09 23:35:55 +0000
@@ -37,6 +37,8 @@
 from bzrlib.index import *
 from bzrlib.knit import (
     AnnotatedKnitContent,
+    DATA_SUFFIX,
+    INDEX_SUFFIX,
     KnitContent,
     KnitGraphIndex,
     KnitVersionedFile,
@@ -45,6 +47,7 @@
     _KnitAccess,
     _KnitData,
     _KnitIndex,
+    make_file_knit,
     _PackAccess,
     PlainKnitContent,
     _StreamAccess,
@@ -53,6 +56,7 @@
     KnitSequenceMatcher,
     )
 from bzrlib.osutils import split_lines
+from bzrlib.symbol_versioning import one_four
 from bzrlib.tests import (
     Feature,
     TestCase,
@@ -158,17 +162,6 @@
         self.assertEqual(content.annotate(),
             [("bogus", "text1"), ("bogus", "text2")])
 
-    def test_annotate_iter(self):
-        content = self._make_content([])
-        it = content.annotate_iter()
-        self.assertRaises(StopIteration, it.next)
-
-        content = self._make_content([("bogus", "text1"), ("bogus", "text2")])
-        it = content.annotate_iter()
-        self.assertEqual(it.next(), ("bogus", "text1"))
-        self.assertEqual(it.next(), ("bogus", "text2"))
-        self.assertRaises(StopIteration, it.next)
-
     def test_line_delta(self):
         content1 = self._make_content([("", "a"), ("", "b")])
         content2 = self._make_content([("", "a"), ("", "a"), ("", "c")])
@@ -196,17 +189,6 @@
         self.assertEqual(content.annotate(),
             [("origin1", "text1"), ("origin2", "text2")])
 
-    def test_annotate_iter(self):
-        content = self._make_content([])
-        it = content.annotate_iter()
-        self.assertRaises(StopIteration, it.next)
-
-        content = self._make_content([("origin1", "text1"), ("origin2", "text2")])
-        it = content.annotate_iter()
-        self.assertEqual(it.next(), ("origin1", "text1"))
-        self.assertEqual(it.next(), ("origin2", "text2"))
-        self.assertRaises(StopIteration, it.next)
-
     def test_line_delta(self):
         content1 = self._make_content([("", "a"), ("", "b")])
         content2 = self._make_content([("", "a"), ("", "a"), ("", "c")])
@@ -492,7 +474,7 @@
         self.addCleanup(reset)
         from bzrlib._knit_load_data_py import _load_data_py
         knit._load_data = _load_data_py
-        return _KnitIndex(*args, **kwargs)
+        return _KnitIndex(get_scope=lambda:None, *args, **kwargs)
 
     def test_no_such_file(self):
         transport = MockTransport()
@@ -677,36 +659,6 @@
         self.assertRaises(RevisionNotPresent,
             index.get_ancestry_with_ghosts, ["e"])
 
-    def test_iter_parents(self):
-        transport = MockTransport()
-        index = self.get_knit_index(transport, "filename", "w", create=True)
-        # no parents
-        index.add_version('r0', ['option'], (None, 0, 1), [])
-        # 1 parent
-        index.add_version('r1', ['option'], (None, 0, 1), ['r0'])
-        # 2 parents
-        index.add_version('r2', ['option'], (None, 0, 1), ['r1', 'r0'])
-        # XXX TODO a ghost
-        # cases: each sample data individually:
-        self.assertEqual(set([('r0', ())]),
-            set(index.iter_parents(['r0'])))
-        self.assertEqual(set([('r1', ('r0', ))]),
-            set(index.iter_parents(['r1'])))
-        self.assertEqual(set([('r2', ('r1', 'r0'))]),
-            set(index.iter_parents(['r2'])))
-        # no nodes returned for a missing node
-        self.assertEqual(set(),
-            set(index.iter_parents(['missing'])))
-        # 1 node returned with missing nodes skipped
-        self.assertEqual(set([('r1', ('r0', ))]),
-            set(index.iter_parents(['ghost1', 'r1', 'ghost'])))
-        # 2 nodes returned
-        self.assertEqual(set([('r0', ()), ('r1', ('r0', ))]),
-            set(index.iter_parents(['r0', 'r1'])))
-        # 2 nodes returned, missing skipped
-        self.assertEqual(set([('r0', ()), ('r1', ('r0', ))]),
-            set(index.iter_parents(['a', 'r0', 'b', 'r1', 'c'])))
-
     def test_num_versions(self):
         transport = MockTransport([
             _KnitIndex.HEADER
@@ -1075,22 +1027,28 @@
         self.addCleanup(reset)
         from bzrlib._knit_load_data_c import _load_data_c
         knit._load_data = _load_data_c
-        return _KnitIndex(*args, **kwargs)
-
+        return _KnitIndex(get_scope=lambda:None, *args, **kwargs)
 
 
 class KnitTests(TestCaseWithTransport):
     """Class containing knit test helper routines."""
 
     def make_test_knit(self, annotate=False, delay_create=False, index=None,
-                       name='test'):
+                       name='test', delta=True, access_mode='w'):
         if not annotate:
             factory = KnitPlainFactory()
         else:
             factory = None
-        return KnitVersionedFile(name, get_transport('.'), access_mode='w',
-                                 factory=factory, create=True,
-                                 delay_create=delay_create, index=index)
+        if index is None:
+            index = _KnitIndex(get_transport('.'), name + INDEX_SUFFIX,
+                access_mode, create=True, file_mode=None,
+                create_parent_dir=False, delay_create=delay_create,
+                dir_mode=None, get_scope=lambda:None)
+        access = _KnitAccess(get_transport('.'), name + DATA_SUFFIX, None,
+            None, delay_create, False)
+        return KnitVersionedFile(name, get_transport('.'), factory=factory,
+            create=True, delay_create=delay_create, index=index,
+            access_method=access, delta=delta)
 
     def assertRecordContentEqual(self, knit, version_id, candidate_content):
         """Assert that some raw record content matches the raw record content
@@ -1115,7 +1073,7 @@
     def test_make_explicit_index(self):
         """We can supply an index to use."""
         knit = KnitVersionedFile('test', get_transport('.'),
-            index='strangelove')
+            index='strangelove', access_method="a")
         self.assertEqual(knit._index, 'strangelove')
 
     def test_knit_add(self):
@@ -1162,7 +1120,8 @@
         k = self.make_test_knit()
         k.add_lines('text-1', [], split_lines(TEXT_1))
         del k
-        k2 = KnitVersionedFile('test', get_transport('.'), access_mode='r', factory=KnitPlainFactory(), create=True)
+        k2 = make_file_knit('test', get_transport('.'), access_mode='r',
+            factory=KnitPlainFactory(), create=True)
         self.assertTrue(k2.has_version('text-1'))
         self.assertEqualDiff(''.join(k2.get_lines('text-1')), TEXT_1)
 
@@ -1190,11 +1149,11 @@
     def test_incomplete(self):
         """Test if texts without a ending line-end can be inserted and
         extracted."""
-        k = KnitVersionedFile('test', get_transport('.'), delta=False, create=True)
+        k = make_file_knit('test', get_transport('.'), delta=False, create=True)
         k.add_lines('text-1', [], ['a\n',    'b'  ])
         k.add_lines('text-2', ['text-1'], ['a\rb\n', 'b\n'])
         # reopening ensures maximum room for confusion
-        k = KnitVersionedFile('test', get_transport('.'), delta=False, create=True)
+        k = make_file_knit('test', get_transport('.'), delta=False, create=True)
         self.assertEquals(k.get_lines('text-1'), ['a\n',    'b'  ])
         self.assertEquals(k.get_lines('text-2'), ['a\rb\n', 'b\n'])
 
@@ -1222,10 +1181,8 @@
 
     def test_add_delta(self):
         """Store in knit with parents"""
-        k = KnitVersionedFile('test', get_transport('.'), factory=KnitPlainFactory(),
-            delta=True, create=True)
+        k = self.make_test_knit(annotate=False)
         self.add_stock_one_and_one_a(k)
-        k.clear_cache()
         self.assertEqualDiff(''.join(k.get_lines('text-1a')), TEXT_1A)
 
     def test_add_delta_knit_graph_index(self):
@@ -1233,10 +1190,8 @@
         index = InMemoryGraphIndex(2)
         knit_index = KnitGraphIndex(index, add_callback=index.add_nodes,
             deltas=True)
-        k = KnitVersionedFile('test', get_transport('.'),
-            delta=True, create=True, index=knit_index)
+        k = self.make_test_knit(annotate=True, index=knit_index)
         self.add_stock_one_and_one_a(k)
-        k.clear_cache()
         self.assertEqualDiff(''.join(k.get_lines('text-1a')), TEXT_1A)
         # check the index had the right data added.
         self.assertEqual(set([
@@ -1248,8 +1203,7 @@
 
     def test_annotate(self):
         """Annotations"""
-        k = KnitVersionedFile('knit', get_transport('.'), factory=KnitAnnotateFactory(),
-            delta=True, create=True)
+        k = self.make_test_knit(annotate=True, name='knit')
         self.insert_and_test_small_annotate(k)
 
     def insert_and_test_small_annotate(self, k):
@@ -1263,8 +1217,7 @@
 
     def test_annotate_fulltext(self):
         """Annotations"""
-        k = KnitVersionedFile('knit', get_transport('.'), factory=KnitAnnotateFactory(),
-            delta=False, create=True)
+        k = self.make_test_knit(annotate=True, name='knit', delta=False)
         self.insert_and_test_small_annotate(k)
 
     def test_annotate_merge_1(self):
@@ -1341,13 +1294,13 @@
         self.assertEquals(origins[2], ('text-1', 'c\n'))
 
     def _test_join_with_factories(self, k1_factory, k2_factory):
-        k1 = KnitVersionedFile('test1', get_transport('.'), factory=k1_factory, create=True)
+        k1 = make_file_knit('test1', get_transport('.'), factory=k1_factory, create=True)
         k1.add_lines('text-a', [], ['a1\n', 'a2\n', 'a3\n'])
         k1.add_lines('text-b', ['text-a'], ['a1\n', 'b2\n', 'a3\n'])
         k1.add_lines('text-c', [], ['c1\n', 'c2\n', 'c3\n'])
         k1.add_lines('text-d', ['text-c'], ['c1\n', 'd2\n', 'd3\n'])
         k1.add_lines('text-m', ['text-b', 'text-d'], ['a1\n', 'b2\n', 'd3\n'])
-        k2 = KnitVersionedFile('test2', get_transport('.'), factory=k2_factory, create=True)
+        k2 = make_file_knit('test2', get_transport('.'), factory=k2_factory, create=True)
         count = k2.join(k1, version_ids=['text-m'])
         self.assertEquals(count, 5)
         self.assertTrue(k2.has_version('text-a'))
@@ -1374,14 +1327,14 @@
         self._test_join_with_factories(KnitPlainFactory(), None)
 
     def test_reannotate(self):
-        k1 = KnitVersionedFile('knit1', get_transport('.'),
+        k1 = make_file_knit('knit1', get_transport('.'),
                                factory=KnitAnnotateFactory(), create=True)
         # 0
         k1.add_lines('text-a', [], ['a\n', 'b\n'])
         # 1
         k1.add_lines('text-b', ['text-a'], ['a\n', 'c\n'])
 
-        k2 = KnitVersionedFile('test2', get_transport('.'),
+        k2 = make_file_knit('test2', get_transport('.'),
                                factory=KnitAnnotateFactory(), create=True)
         k2.join(k1, version_ids=['text-b'])
 
@@ -1404,8 +1357,8 @@
 
     def test_get_line_delta_texts(self):
         """Make sure we can call get_texts on text with reused line deltas"""
-        k1 = KnitVersionedFile('test1', get_transport('.'), 
-                               factory=KnitPlainFactory(), create=True)
+        k1 = make_file_knit('test1', get_transport('.'),
+            factory=KnitPlainFactory(), create=True)
         for t in range(3):
             if t == 0:
                 parents = []
@@ -1416,12 +1369,11 @@
         
     def test_iter_lines_reads_in_order(self):
         instrumented_t = get_transport('trace+memory:///')
-        k1 = KnitVersionedFile('id', instrumented_t, create=True, delta=True)
+        k1 = make_file_knit('id', instrumented_t, create=True, delta=True)
         self.assertEqual([('get', 'id.kndx',)], instrumented_t._activity)
         # add texts with no required ordering
         k1.add_lines('base', [], ['text\n'])
         k1.add_lines('base2', [], ['text2\n'])
-        k1.clear_cache()
         # clear the logged activity, but preserve the list instance in case of
         # clones pointing at it.
         del instrumented_t._activity[:]
@@ -1452,19 +1404,19 @@
             "\nrevid2 line-delta 84 82 0 :",
             'test.kndx')
         # we should be able to load this file again
-        knit = KnitVersionedFile('test', get_transport('.'), access_mode='r')
+        knit = make_file_knit('test', get_transport('.'), access_mode='r')
         self.assertEqual(['revid', 'revid2'], knit.versions())
         # write a short write to the file and ensure that its ignored
         indexfile = file('test.kndx', 'ab')
         indexfile.write('\nrevid3 line-delta 166 82 1 2 3 4 5 .phwoar:demo ')
         indexfile.close()
         # we should be able to load this file again
-        knit = KnitVersionedFile('test', get_transport('.'), access_mode='w')
+        knit = make_file_knit('test', get_transport('.'), access_mode='w')
         self.assertEqual(['revid', 'revid2'], knit.versions())
         # and add a revision with the same id the failed write had
         knit.add_lines('revid3', ['revid2'], ['a\n'])
         # and when reading it revid3 should now appear.
-        knit = KnitVersionedFile('test', get_transport('.'), access_mode='r')
+        knit = make_file_knit('test', get_transport('.'), access_mode='r')
         self.assertEqual(['revid', 'revid2', 'revid3'], knit.versions())
         self.assertEqual({'revid3':('revid2',)}, knit.get_parent_map(['revid3']))
 
@@ -1485,11 +1437,11 @@
         """create_parent_dir can create knits in nonexistant dirs"""
         # Has no effect if we don't set 'delay_create'
         trans = get_transport('.')
-        self.assertRaises(NoSuchFile, KnitVersionedFile, 'dir/test',
+        self.assertRaises(NoSuchFile, make_file_knit, 'dir/test',
                           trans, access_mode='w', factory=None,
                           create=True, create_parent_dir=True)
         # Nothing should have changed yet
-        knit = KnitVersionedFile('dir/test', trans, access_mode='w',
+        knit = make_file_knit('dir/test', trans, access_mode='w',
                                  factory=None, create=True,
                                  create_parent_dir=True,
                                  delay_create=True)
@@ -1510,12 +1462,9 @@
         if not trans._can_roundtrip_unix_modebits():
             # Can't roundtrip, so no need to run this test
             return
-        knit = KnitVersionedFile('dir/test', trans, access_mode='w',
-                                 factory=None, create=True,
-                                 create_parent_dir=True,
-                                 delay_create=True,
-                                 file_mode=0600,
-                                 dir_mode=0700)
+        knit = make_file_knit('dir/test', trans, access_mode='w', factory=None,
+            create=True, create_parent_dir=True, delay_create=True,
+            file_mode=0600, dir_mode=0700)
         knit.add_lines('revid', [], ['a\n'])
         self.assertTransportMode(trans, 'dir', 0700)
         self.assertTransportMode(trans, 'dir/test.knit', 0600)
@@ -1526,12 +1475,9 @@
         if not trans._can_roundtrip_unix_modebits():
             # Can't roundtrip, so no need to run this test
             return
-        knit = KnitVersionedFile('dir/test', trans, access_mode='w',
-                                 factory=None, create=True,
-                                 create_parent_dir=True,
-                                 delay_create=True,
-                                 file_mode=0660,
-                                 dir_mode=0770)
+        knit = make_file_knit('dir/test', trans, access_mode='w', factory=None,
+            create=True, create_parent_dir=True, delay_create=True,
+            file_mode=0660, dir_mode=0770)
         knit.add_lines('revid', [], ['a\n'])
         self.assertTransportMode(trans, 'dir', 0770)
         self.assertTransportMode(trans, 'dir/test.knit', 0660)
@@ -1542,12 +1488,9 @@
         if not trans._can_roundtrip_unix_modebits():
             # Can't roundtrip, so no need to run this test
             return
-        knit = KnitVersionedFile('dir/test', trans, access_mode='w',
-                                 factory=None, create=True,
-                                 create_parent_dir=True,
-                                 delay_create=True,
-                                 file_mode=0666,
-                                 dir_mode=0777)
+        knit = make_file_knit('dir/test', trans, access_mode='w', factory=None,
+            create=True, create_parent_dir=True, delay_create=True,
+            file_mode=0666, dir_mode=0777)
         knit.add_lines('revid', [], ['a\n'])
         self.assertTransportMode(trans, 'dir', 0777)
         self.assertTransportMode(trans, 'dir/test.knit', 0666)
@@ -2087,7 +2030,7 @@
     def test_weave_to_knit_matches(self):
         # check that the WeaveToKnit is_compatible function
         # registers True for a Weave to a Knit.
-        w = Weave()
+        w = Weave(get_scope=lambda:None)
         k = self.make_test_knit()
         self.failUnless(WeaveToKnit.is_compatible(w, k))
         self.failIf(WeaveToKnit.is_compatible(k, w))
@@ -2095,87 +2038,6 @@
         self.failIf(WeaveToKnit.is_compatible(k, k))
 
 
-class TestKnitCaching(KnitTests):
-    
-    def create_knit(self):
-        k = self.make_test_knit(True)
-        k.add_lines('text-1', [], split_lines(TEXT_1))
-        k.add_lines('text-2', [], split_lines(TEXT_2))
-        return k
-
-    def test_no_caching(self):
-        k = self.create_knit()
-        # Nothing should be cached without setting 'enable_cache'
-        self.assertEqual({}, k._data._cache)
-
-    def test_cache_data_read_raw(self):
-        k = self.create_knit()
-
-        # Now cache and read
-        k.enable_cache()
-
-        def read_one_raw(version):
-            pos_map = k._get_components_positions([version])
-            method, index_memo, next = pos_map[version]
-            lst = list(k._data.read_records_iter_raw([(version, index_memo)]))
-            self.assertEqual(1, len(lst))
-            return lst[0]
-
-        val = read_one_raw('text-1')
-        self.assertEqual({'text-1':val[1]}, k._data._cache)
-
-        k.clear_cache()
-        # After clear, new reads are not cached
-        self.assertEqual({}, k._data._cache)
-
-        val2 = read_one_raw('text-1')
-        self.assertEqual(val, val2)
-        self.assertEqual({}, k._data._cache)
-
-    def test_cache_data_read(self):
-        k = self.create_knit()
-
-        def read_one(version):
-            pos_map = k._get_components_positions([version])
-            method, index_memo, next = pos_map[version]
-            lst = list(k._data.read_records_iter([(version, index_memo)]))
-            self.assertEqual(1, len(lst))
-            return lst[0]
-
-        # Now cache and read
-        k.enable_cache()
-
-        val = read_one('text-2')
-        self.assertEqual(['text-2'], k._data._cache.keys())
-        self.assertEqual('text-2', val[0])
-        content, digest = k._data._parse_record('text-2',
-                                                k._data._cache['text-2'])
-        self.assertEqual(content, val[1])
-        self.assertEqual(digest, val[2])
-
-        k.clear_cache()
-        self.assertEqual({}, k._data._cache)
-
-        val2 = read_one('text-2')
-        self.assertEqual(val, val2)
-        self.assertEqual({}, k._data._cache)
-
-    def test_cache_read(self):
-        k = self.create_knit()
-        k.enable_cache()
-
-        text = k.get_text('text-1')
-        self.assertEqual(TEXT_1, text)
-        self.assertEqual(['text-1'], k._data._cache.keys())
-
-        k.clear_cache()
-        self.assertEqual({}, k._data._cache)
-
-        text = k.get_text('text-1')
-        self.assertEqual(TEXT_1, text)
-        self.assertEqual({}, k._data._cache)
-
-
 class TestKnitIndex(KnitTests):
 
     def test_add_versions_dictionary_compresses(self):
@@ -2516,40 +2378,6 @@
              ('tip', 'no-eol,line-delta', (None, 0, 100), ['parent'])])
         self.assertEqual([], self.caught_entries)
 
-    def test_iter_parents(self):
-        index1 = self.make_g_index('1', 1, [
-        # no parents
-            (('r0', ), 'N0 100', ([], )),
-        # 1 parent
-            (('r1', ), '', ([('r0', )], ))])
-        index2 = self.make_g_index('2', 1, [
-        # 2 parents
-            (('r2', ), 'N0 100', ([('r1', ), ('r0', )], )),
-            ])
-        combined_index = CombinedGraphIndex([index1, index2])
-        index = KnitGraphIndex(combined_index)
-        # XXX TODO a ghost
-        # cases: each sample data individually:
-        self.assertEqual(set([('r0', ())]),
-            set(index.iter_parents(['r0'])))
-        self.assertEqual(set([('r1', ('r0', ))]),
-            set(index.iter_parents(['r1'])))
-        self.assertEqual(set([('r2', ('r1', 'r0'))]),
-            set(index.iter_parents(['r2'])))
-        # no nodes returned for a missing node
-        self.assertEqual(set(),
-            set(index.iter_parents(['missing'])))
-        # 1 node returned with missing nodes skipped
-        self.assertEqual(set([('r1', ('r0', ))]),
-            set(index.iter_parents(['ghost1', 'r1', 'ghost'])))
-        # 2 nodes returned
-        self.assertEqual(set([('r0', ()), ('r1', ('r0', ))]),
-            set(index.iter_parents(['r0', 'r1'])))
-        # 2 nodes returned, missing skipped
-        self.assertEqual(set([('r0', ()), ('r1', ('r0', ))]),
-            set(index.iter_parents(['a', 'r0', 'b', 'r1', 'c'])))
-
-
 class TestNoParentsGraphIndexKnit(KnitTests):
     """Tests for knits using KnitGraphIndex with no parents."""
 
@@ -2767,18 +2595,6 @@
              ('tip', 'no-eol,line-delta', (None, 0, 100), [])])
         self.assertEqual([], self.caught_entries)
 
-    def test_iter_parents(self):
-        index = self.two_graph_index()
-        self.assertEqual(set([
-            ('tip', ()), ('tail', ()), ('parent', ()), ('separate', ())
-            ]),
-            set(index.iter_parents(['tip', 'tail', 'ghost', 'parent', 'separate'])))
-        self.assertEqual(set([('tip', ())]),
-            set(index.iter_parents(['tip'])))
-        self.assertEqual(set(),
-            set(index.iter_parents([])))
-
-
 class TestPackKnits(KnitTests):
     """Tests that use a _PackAccess and KnitGraphIndex."""
 
@@ -2833,11 +2649,6 @@
             set(result),
             set(index.get_ancestry(ancestry_versions, False)))
 
-    def assertIterParents(self, knit, versions, parent_versions, result):
-        """Check the result of an iter_parents call on knit."""
-        index = self.get_index(knit, knit.get_data_stream(versions))
-        self.assertEqual(result, index.iter_parents(parent_versions))
-
     def assertGetMethod(self, knit, versions, version, result):
         index = self.get_index(knit, knit.get_data_stream(versions))
         self.assertEqual(result, index.get_method(version))
@@ -2908,19 +2719,6 @@
         # we thunk across.
         self.assertGetMethod(knit, ['c'], 'b', 'fulltext')
 
-    def test_iter_parents(self):
-        knit = self.make_knit_with_4_versions_2_dags()
-        self.assertIterParents(knit, ['a'], ['a'], [('a', ())])
-        self.assertIterParents(knit, ['a', 'b'], ['a', 'b'],
-            [('a', ()), ('b', ())])
-        self.assertIterParents(knit, ['a', 'b', 'c'], ['a', 'b', 'c'],
-            [('a', ()), ('b', ()), ('c', ('b', 'a'))])
-        self.assertIterParents(knit, ['a', 'b', 'c', 'd'],
-            ['a', 'b', 'c', 'd'],
-            [('a', ()), ('b', ()), ('c', ('b', 'a')), ('d', ('e', 'f'))])
-        self.assertIterParents(knit, ['c'], ['a', 'b', 'c'],
-            [('c', ('b', 'a'))])
-
     def test_get_options(self):
         knit = self.make_knit_with_4_versions_2_dags()
         self.assertGetOptions(knit, 'a', ['no-eol', 'fulltext'])
@@ -3012,3 +2810,13 @@
             knit.get_data_stream([]))
         self.assertRaises(errors.KnitCorrupt,
             list, access.get_raw_records([(True, "A", None, None)]))
+
+
+class TestFormatSignatures(KnitTests):
+
+    def test_knit_format_signatures(self):
+        """Different formats of knit have different signature strings."""
+        knit = self.make_test_knit(name='a', annotate=True)
+        self.assertEqual('knit-annotated', knit.get_format_signature())
+        knit = self.make_test_knit(name='p', annotate=False)
+        self.assertEqual('knit-plain', knit.get_format_signature())

=== modified file 'bzrlib/tests/test_merge.py'
--- bzrlib/tests/test_merge.py	2008-04-04 04:20:19 +0000
+++ bzrlib/tests/test_merge.py	2008-04-07 04:54:52 +0000
@@ -469,8 +469,8 @@
 
     def setUp(self):
         TestCaseWithMemoryTransport.setUp(self)
-        self.vf = knit.KnitVersionedFile('root', self.get_transport(),
-                                         create=True)
+        self.vf = knit.make_file_knit('root', self.get_transport(),
+            create=True)
         self.plan_merge_vf = versionedfile._PlanMergeVersionedFile('root',
                                                                    [self.vf])
 

=== modified file 'bzrlib/tests/test_store.py'
--- bzrlib/tests/test_store.py	2007-08-15 04:33:34 +0000
+++ bzrlib/tests/test_store.py	2008-04-08 00:57:07 +0000
@@ -405,32 +405,32 @@
 
 class TestVersionFileStore(TestCaseWithTransport):
 
+    def get_scope(self):
+        return self._transaction
+
     def setUp(self):
         super(TestVersionFileStore, self).setUp()
         self.vfstore = store.versioned.VersionedFileStore(MemoryTransport())
+        self.vfstore.get_scope = self.get_scope
+        self._transaction = None
 
     def test_get_weave_registers_dirty_in_write(self):
-        transaction = transactions.WriteTransaction()
-        vf = self.vfstore.get_weave_or_empty('id', transaction)
-        transaction.finish()
-        self.assertRaises(errors.OutSideTransaction, vf.add_lines, 'b', [], [])
-        transaction = transactions.WriteTransaction()
-        vf = self.vfstore.get_weave('id', transaction)
-        transaction.finish()
-        self.assertRaises(errors.OutSideTransaction, vf.add_lines, 'b', [], [])
-
-    def test_get_weave_or_empty_readonly_fails(self):
-        transaction = transactions.ReadOnlyTransaction()
-        vf = self.assertRaises(errors.ReadOnlyError,
-                               self.vfstore.get_weave_or_empty,
-                               'id',
-                               transaction)
+        self._transaction = transactions.WriteTransaction()
+        vf = self.vfstore.get_weave_or_empty('id', self._transaction)
+        self._transaction.finish()
+        self._transaction = None
+        self.assertRaises(errors.OutSideTransaction, vf.add_lines, 'b', [], [])
+        self._transaction = transactions.WriteTransaction()
+        vf = self.vfstore.get_weave('id', self._transaction)
+        self._transaction.finish()
+        self._transaction = None
+        self.assertRaises(errors.OutSideTransaction, vf.add_lines, 'b', [], [])
 
     def test_get_weave_readonly_cant_write(self):
-        transaction = transactions.WriteTransaction()
-        vf = self.vfstore.get_weave_or_empty('id', transaction)
-        transaction.finish()
-        transaction = transactions.ReadOnlyTransaction()
-        vf = self.vfstore.get_weave_or_empty('id', transaction)
+        self._transaction = transactions.WriteTransaction()
+        vf = self.vfstore.get_weave_or_empty('id', self._transaction)
+        self._transaction.finish()
+        self._transaction = transactions.ReadOnlyTransaction()
+        vf = self.vfstore.get_weave_or_empty('id', self._transaction)
         self.assertRaises(errors.ReadOnlyError, vf.add_lines, 'b', [], [])
 

=== modified file 'bzrlib/tests/test_subsume.py'
--- bzrlib/tests/test_subsume.py	2007-03-06 08:41:40 +0000
+++ bzrlib/tests/test_subsume.py	2008-04-04 00:06:58 +0000
@@ -22,7 +22,7 @@
     def make_branch_and_tree(self, relpath, format=None):
         if format is None:
             format = 'dirstate-with-subtree'
-        return tests.TestCaseWithTransport.make_branch_and_tree(self, relpath, 
+        return tests.TestCaseWithTransport.make_branch_and_tree(self, relpath,
                                                                 format)
 
     def make_trees(self, format=None, same_root=False):

=== modified file 'bzrlib/tests/test_versionedfile.py'
--- bzrlib/tests/test_versionedfile.py	2008-04-01 02:41:25 +0000
+++ bzrlib/tests/test_versionedfile.py	2008-04-08 21:41:15 +0000
@@ -30,12 +30,12 @@
     progress,
     )
 from bzrlib.errors import (
-                           RevisionNotPresent, 
+                           RevisionNotPresent,
                            RevisionAlreadyPresent,
                            WeaveParentMismatch
                            )
 from bzrlib.knit import (
-    KnitVersionedFile,
+    make_file_knit,
     KnitAnnotateFactory,
     KnitPlainFactory,
     )
@@ -59,6 +59,11 @@
     they are strictly controlled by their owning repositories.
     """
 
+    def get_transaction(self):
+        if not hasattr(self, '_transaction'):
+            self._transaction = None
+        return self._transaction
+
     def test_add(self):
         f = self.get_file()
         f.add_lines('r0', [], ['a\n', 'b\n'])
@@ -242,7 +247,7 @@
         for version in multiparent.topo_iter(vf):
             mpdiff = vf.make_mpdiffs([version])[0]
             new_vf.add_mpdiffs([(version, vf.get_parent_map([version])[version],
-                                 vf.get_sha1(version), mpdiff)])
+                                 vf.get_sha1s([version])[0], mpdiff)])
             self.assertEqualDiff(vf.get_text(version),
                                  new_vf.get_text(version))
 
@@ -348,34 +353,25 @@
             set(f.get_ancestry('rM', topo_sorted=False)))
 
     def test_mutate_after_finish(self):
+        self._transaction = 'before'
         f = self.get_file()
-        f.transaction_finished()
+        self._transaction = 'after'
         self.assertRaises(errors.OutSideTransaction, f.add_lines, '', [], [])
         self.assertRaises(errors.OutSideTransaction, f.add_lines_with_ghosts, '', [], [])
         self.assertRaises(errors.OutSideTransaction, f.join, '')
-        self.assertRaises(errors.OutSideTransaction, f.clone_text, 'base', 'bar', ['foo'])
         
-    def test_clear_cache(self):
-        f = self.get_file()
-        # on a new file it should not error
-        f.clear_cache()
-        # and after adding content, doing a clear_cache and a get should work.
-        f.add_lines('0', [], ['a'])
-        f.clear_cache()
-        self.assertEqual(['a'], f.get_lines('0'))
-
     def test_clone_text(self):
         f = self.get_file()
         f.add_lines('r0', [], ['a\n', 'b\n'])
-        f.clone_text('r1', 'r0', ['r0'])
+        self.applyDeprecated(one_four, f.clone_text, 'r1', 'r0', ['r0'])
         def verify_file(f):
             self.assertEquals(f.get_lines('r1'), f.get_lines('r0'))
             self.assertEquals(f.get_lines('r1'), ['a\n', 'b\n'])
             self.assertEqual({'r1':('r0',)}, f.get_parent_map(['r1']))
             self.assertRaises(RevisionNotPresent,
-                f.clone_text, 'r2', 'rX', [])
+                self.applyDeprecated, one_four, f.clone_text, 'r2', 'rX', [])
             self.assertRaises(RevisionAlreadyPresent,
-                f.clone_text, 'r1', 'r0', [])
+                self.applyDeprecated, one_four, f.clone_text, 'r1', 'r0', [])
         verify_file(f)
         verify_file(self.reopen_file())
 
@@ -384,15 +380,13 @@
         f.add_lines('0', [], ['a\n'])
         t = MemoryTransport()
         f.copy_to('foo', t)
-        for suffix in f.__class__.get_suffixes():
+        for suffix in self.get_factory().get_suffixes():
             self.assertTrue(t.has('foo' + suffix))
 
     def test_get_suffixes(self):
         f = self.get_file()
-        # should be the same
-        self.assertEqual(f.__class__.get_suffixes(), f.__class__.get_suffixes())
         # and should be a list
-        self.assertTrue(isinstance(f.__class__.get_suffixes(), list))
+        self.assertTrue(isinstance(self.get_factory().get_suffixes(), list))
 
     def build_graph(self, file, graph):
         for node in topo_sort(graph.items()):
@@ -405,7 +399,7 @@
             'v2': ('v1', ),
             'v3': ('v2', )}
         self.build_graph(f, graph)
-        self.assertEqual(graph, f.get_graph())
+        self.assertEqual(graph, self.applyDeprecated(one_four, f.get_graph))
     
     def test_get_graph_partial(self):
         f = self.get_file()
@@ -432,10 +426,14 @@
         simple_b_gam.update(simple_gam)
         simple_b_gam.update(simple_b)
         self.build_graph(f, complex_graph)
-        self.assertEqual(simple_a, f.get_graph(['a']))
-        self.assertEqual(simple_b, f.get_graph(['b']))
-        self.assertEqual(simple_gam, f.get_graph(['gam']))
-        self.assertEqual(simple_b_gam, f.get_graph(['b', 'gam']))
+        self.assertEqual(simple_a, self.applyDeprecated(one_four, f.get_graph,
+            ['a']))
+        self.assertEqual(simple_b, self.applyDeprecated(one_four, f.get_graph,
+            ['b']))
+        self.assertEqual(simple_gam, self.applyDeprecated(one_four,
+            f.get_graph, ['gam']))
+        self.assertEqual(simple_b_gam, self.applyDeprecated(one_four,
+            f.get_graph, ['b', 'gam']))
 
     def test_get_parents(self):
         f = self.get_file()
@@ -525,23 +523,25 @@
         # XXX TODO a ghost
         # cases: each sample data individually:
         self.assertEqual(set([('r0', ())]),
-            set(f.iter_parents(['r0'])))
+            set(self.applyDeprecated(one_four, f.iter_parents, ['r0'])))
         self.assertEqual(set([('r1', ('r0', ))]),
-            set(f.iter_parents(['r1'])))
+            set(self.applyDeprecated(one_four, f.iter_parents, ['r1'])))
         self.assertEqual(set([('r2', ('r1', 'r0'))]),
-            set(f.iter_parents(['r2'])))
+            set(self.applyDeprecated(one_four, f.iter_parents, ['r2'])))
         # no nodes returned for a missing node
         self.assertEqual(set(),
-            set(f.iter_parents(['missing'])))
+            set(self.applyDeprecated(one_four, f.iter_parents, ['missing'])))
         # 1 node returned with missing nodes skipped
         self.assertEqual(set([('r1', ('r0', ))]),
-            set(f.iter_parents(['ghost1', 'r1', 'ghost'])))
+            set(self.applyDeprecated(one_four, f.iter_parents, ['ghost1', 'r1',
+                'ghost'])))
         # 2 nodes returned
         self.assertEqual(set([('r0', ()), ('r1', ('r0', ))]),
-            set(f.iter_parents(['r0', 'r1'])))
+            set(self.applyDeprecated(one_four, f.iter_parents, ['r0', 'r1'])))
         # 2 nodes returned, missing skipped
         self.assertEqual(set([('r0', ()), ('r1', ('r0', ))]),
-            set(f.iter_parents(['a', 'r0', 'b', 'r1', 'c'])))
+            set(self.applyDeprecated(one_four, f.iter_parents,
+                ['a', 'r0', 'b', 'r1', 'c'])))
 
     def test_iter_lines_added_or_present_in_versions(self):
         # test that we get at least an equalset of the lines added by
@@ -631,7 +631,8 @@
         self.assertEqual(['notbxbfse'], vf.get_ancestry('notbxbfse'))
         self.assertEqual([],
             self.applyDeprecated(one_four, vf.get_parents, 'notbxbfse'))
-        self.assertEqual({'notbxbfse':()}, vf.get_graph())
+        self.assertEqual({'notbxbfse':()}, self.applyDeprecated(one_four,
+            vf.get_graph))
         self.assertFalse(vf.has_version(parent_id_utf8))
         # we have _with_ghost apis to give us ghost information.
         self.assertEqual([parent_id_utf8, 'notbxbfse'], vf.get_ancestry_with_ghosts(['notbxbfse']))
@@ -649,7 +650,7 @@
         self.assertEqual({parent_id_utf8:(),
                           'notbxbfse':(parent_id_utf8, ),
                           },
-                         vf.get_graph())
+                         self.applyDeprecated(one_four, vf.get_graph))
         self.assertTrue(vf.has_version(parent_id_utf8))
         # we have _with_ghost apis to give us ghost information.
         self.assertEqual([parent_id_utf8, 'notbxbfse'],
@@ -693,9 +694,8 @@
                           [],
                           [])
         self.assertRaises(errors.ReadOnlyError, vf.join, 'base')
-        self.assertRaises(errors.ReadOnlyError, vf.clone_text, 'base', 'bar', ['foo'])
     
-    def test_get_sha1(self):
+    def test_get_sha1s(self):
         # check the sha1 data is available
         vf = self.get_file()
         # a simple file
@@ -704,13 +704,16 @@
         vf.add_lines('b', ['a'], ['a\n'])
         # a file differing only in last newline.
         vf.add_lines('c', [], ['a'])
-        self.assertEqual(
-            '3f786850e387550fdab836ed7e6dc881de23001b', vf.get_sha1('a'))
-        self.assertEqual(
-            '3f786850e387550fdab836ed7e6dc881de23001b', vf.get_sha1('b'))
-        self.assertEqual(
-            '86f7e437faa5a7fce15d1ddcb9eaeaea377667b8', vf.get_sha1('c'))
-
+        # Deprecasted single-version API.
+        self.assertEqual(
+            '3f786850e387550fdab836ed7e6dc881de23001b',
+            self.applyDeprecated(one_four, vf.get_sha1, 'a'))
+        self.assertEqual(
+            '3f786850e387550fdab836ed7e6dc881de23001b',
+            self.applyDeprecated(one_four, vf.get_sha1, 'b'))
+        self.assertEqual(
+            '86f7e437faa5a7fce15d1ddcb9eaeaea377667b8',
+            self.applyDeprecated(one_four, vf.get_sha1, 'c'))
         self.assertEqual(['3f786850e387550fdab836ed7e6dc881de23001b',
                           '86f7e437faa5a7fce15d1ddcb9eaeaea377667b8',
                           '3f786850e387550fdab836ed7e6dc881de23001b'],
@@ -720,10 +723,12 @@
 class TestWeave(TestCaseWithMemoryTransport, VersionedFileTestMixIn):
 
     def get_file(self, name='foo'):
-        return WeaveFile(name, get_transport(self.get_url('.')), create=True)
+        return WeaveFile(name, get_transport(self.get_url('.')), create=True,
+            get_scope=self.get_transaction)
 
     def get_file_corrupted_text(self):
-        w = WeaveFile('foo', get_transport(self.get_url('.')), create=True)
+        w = WeaveFile('foo', get_transport(self.get_url('.')), create=True,
+            get_scope=self.get_transaction)
         w.add_lines('v1', [], ['hello\n'])
         w.add_lines('v2', ['v1'], ['hello\n', 'there\n'])
         
@@ -757,13 +762,15 @@
         return w
 
     def reopen_file(self, name='foo', create=False):
-        return WeaveFile(name, get_transport(self.get_url('.')), create=create)
+        return WeaveFile(name, get_transport(self.get_url('.')), create=create,
+            get_scope=self.get_transaction)
 
     def test_no_implicit_create(self):
         self.assertRaises(errors.NoSuchFile,
                           WeaveFile,
                           'foo',
-                          get_transport(self.get_url('.')))
+                          get_transport(self.get_url('.')),
+                          get_scope=self.get_transaction)
 
     def get_factory(self):
         return WeaveFile
@@ -773,10 +780,10 @@
 
     def get_file(self, name='foo'):
         return self.get_factory()(name, get_transport(self.get_url('.')),
-                                  delta=True, create=True)
+            delta=True, create=True, get_scope=self.get_transaction)
 
     def get_factory(self):
-        return KnitVersionedFile
+        return make_file_knit
 
     def get_file_corrupted_text(self):
         knit = self.get_file()
@@ -794,31 +801,23 @@
         knit.check()
 
     def test_no_implicit_create(self):
-        self.assertRaises(errors.NoSuchFile,
-                          KnitVersionedFile,
-                          'foo',
-                          get_transport(self.get_url('.')))
+        self.assertRaises(errors.NoSuchFile, self.get_factory(), 'foo',
+            get_transport(self.get_url('.')))
 
 
 class TestPlaintextKnit(TestKnit):
     """Test a knit with no cached annotations"""
 
-    def _factory(self, name, transport, file_mode=None, access_mode=None,
-                 delta=True, create=False):
-        return KnitVersionedFile(name, transport, file_mode, access_mode,
-                                 KnitPlainFactory(), delta=delta,
-                                 create=create)
-
     def get_factory(self):
-        return self._factory
+        return make_file_knit
 
 
 class TestPlanMergeVersionedFile(TestCaseWithMemoryTransport):
 
     def setUp(self):
         TestCaseWithMemoryTransport.setUp(self)
-        self.vf1 = KnitVersionedFile('root', self.get_transport(), create=True)
-        self.vf2 = KnitVersionedFile('root', self.get_transport(), create=True)
+        self.vf1 = make_file_knit('root', self.get_transport(), create=True)
+        self.vf2 = make_file_knit('root', self.get_transport(), create=True)
         self.plan_merge_vf = versionedfile._PlanMergeVersionedFile('root',
             [self.vf1, self.vf2])
 
@@ -946,6 +945,9 @@
 
 class TestReadonlyHttpMixin(object):
 
+    def get_transaction(self):
+        return 1
+
     def test_readonly_http_works(self):
         # we should be able to read from http with a versioned file.
         vf = self.get_file()
@@ -964,7 +966,8 @@
 class TestWeaveHTTP(TestCaseWithWebserver, TestReadonlyHttpMixin):
 
     def get_file(self):
-        return WeaveFile('foo', get_transport(self.get_url('.')), create=True)
+        return WeaveFile('foo', get_transport(self.get_url('.')), create=True,
+            get_scope=self.get_transaction)
 
     def get_factory(self):
         return WeaveFile
@@ -973,11 +976,11 @@
 class TestKnitHTTP(TestCaseWithWebserver, TestReadonlyHttpMixin):
 
     def get_file(self):
-        return KnitVersionedFile('foo', get_transport(self.get_url('.')),
-                                 delta=True, create=True)
+        return make_file_knit('foo', get_transport(self.get_url('.')),
+            delta=True, create=True, get_scope=self.get_transaction)
 
     def get_factory(self):
-        return KnitVersionedFile
+        return make_file_knit
 
 
 class MergeCasesMixin(object):
@@ -1220,7 +1223,7 @@
 class TestKnitMerge(TestCaseWithMemoryTransport, MergeCasesMixin):
 
     def get_file(self, name='foo'):
-        return KnitVersionedFile(name, get_transport(self.get_url('.')),
+        return make_file_knit(name, get_transport(self.get_url('.')),
                                  delta=True, create=True)
 
     def log_contents(self, w):
@@ -1240,23 +1243,3 @@
 
     overlappedInsertExpected = ['aaa', '<<<<<<< ', 'xxx', 'yyy', '=======', 
                                 'xxx', '>>>>>>> ', 'bbb']
-
-
-class TestFormatSignatures(TestCaseWithMemoryTransport):
-
-    def get_knit_file(self, name, annotated):
-        if annotated:
-            factory = KnitAnnotateFactory()
-        else:
-            factory = KnitPlainFactory()
-        return KnitVersionedFile(
-            name, get_transport(self.get_url('.')), create=True,
-            factory=factory)
-
-    def test_knit_format_signatures(self):
-        """Different formats of knit have different signature strings."""
-        knit = self.get_knit_file('a', True)
-        self.assertEqual('knit-annotated', knit.get_format_signature())
-        knit = self.get_knit_file('p', False)
-        self.assertEqual('knit-plain', knit.get_format_signature())
-

=== modified file 'bzrlib/tests/test_weave.py'
--- bzrlib/tests/test_weave.py	2007-09-19 05:14:14 +0000
+++ bzrlib/tests/test_weave.py	2008-04-08 03:39:43 +0000
@@ -65,7 +65,7 @@
 class WeaveContains(TestBase):
     """Weave __contains__ operator"""
     def runTest(self):
-        k = Weave()
+        k = Weave(get_scope=lambda:None)
         self.assertFalse('foo' in k)
         k.add_lines('foo', [], TEXT_1)
         self.assertTrue('foo' in k)
@@ -84,18 +84,6 @@
                          [('text0', TEXT_0[0])])
 
 
-class GetSha1(TestBase):
-    def test_get_sha1(self):
-        k = Weave()
-        k.add_lines('text0', [], 'text0')
-        self.assertEqual('34dc0e430c642a26c3dd1c2beb7a8b4f4445eb79',
-                         k.get_sha1('text0'))
-        self.assertRaises(errors.RevisionNotPresent,
-                          k.get_sha1, 0)
-        self.assertRaises(errors.RevisionNotPresent,
-                          k.get_sha1, 'text1')
-                        
-
 class InvalidAdd(TestBase):
     """Try to use invalid version number during add."""
     def runTest(self):

=== modified file 'bzrlib/versionedfile.py'
--- bzrlib/versionedfile.py	2008-04-01 02:41:25 +0000
+++ bzrlib/versionedfile.py	2008-04-09 00:34:54 +0000
@@ -55,10 +55,6 @@
     Texts are identified by a version-id string.
     """
 
-    def __init__(self, access_mode):
-        self.finished = False
-        self._access_mode = access_mode
-
     @staticmethod
     def check_not_reserved_id(version_id):
         revision.check_not_reserved_id(version_id)
@@ -161,13 +157,7 @@
             if '\n' in line[:-1]:
                 raise errors.BzrBadParameterContainsNewline("lines")
 
-    def _check_write_ok(self):
-        """Is the versioned file marked as 'finished' ? Raise if it is."""
-        if self.finished:
-            raise errors.OutSideTransaction()
-        if self._access_mode != 'w':
-            raise errors.ReadOnlyObjectDirtiedError(self)
-
+    @deprecated_method(one_four)
     def enable_cache(self):
         """Tell this versioned file that it should cache any data it reads.
         
@@ -175,6 +165,7 @@
         """
         pass
     
+    @deprecated_method(one_four)
     def clear_cache(self):
         """Remove any data cached in the versioned file object.
 
@@ -182,6 +173,7 @@
         """
         pass
 
+    @deprecated_method(one_four)
     def clone_text(self, new_version_id, old_version_id, parents):
         """Add an identical text to old_version_id as new_version_id.
 
@@ -191,11 +183,8 @@
         Must raise RevisionAlreadyPresent if the new version is
         already present in file history."""
         self._check_write_ok()
-        return self._clone_text(new_version_id, old_version_id, parents)
-
-    def _clone_text(self, new_version_id, old_version_id, parents):
-        """Helper function to do the _clone_text work."""
-        raise NotImplementedError(self.clone_text)
+        return self.add_lines(new_version_id, parents,
+            self.get_lines(old_version_id))
 
     def get_format_signature(self):
         """Get a text description of the data encoding in this file.
@@ -282,12 +271,13 @@
             if expected_sha1 != sha1:
                 raise errors.VersionedFileInvalidChecksum(version)
 
+    @deprecated_method(one_four)
     def get_sha1(self, version_id):
         """Get the stored sha1 sum for the given revision.
         
         :param version_id: The name of the version to lookup
         """
-        raise NotImplementedError(self.get_sha1)
+        return self.get_sha1s([version_id])[0]
 
     def get_sha1s(self, version_ids):
         """Get the stored sha1 sums for the given revisions.
@@ -297,10 +287,6 @@
         """
         raise NotImplementedError(self.get_sha1s)
 
-    def get_suffixes(self):
-        """Return the file suffixes associated with this versioned file."""
-        raise NotImplementedError(self.get_suffixes)
-    
     def get_text(self, version_id):
         """Return version contents as a text string.
 
@@ -353,7 +339,8 @@
         but are not explicitly marked.
         """
         raise NotImplementedError(self.get_ancestry_with_ghosts)
-        
+    
+    @deprecated_method(one_four)
     def get_graph(self, version_ids=None):
         """Return a graph from the versioned file. 
         
@@ -362,18 +349,28 @@
                             None means retrieve all versions.
         """
         if version_ids is None:
-            return dict(self.iter_parents(self.versions()))
-        result = {}
-        pending = set(version_ids)
-        while pending:
-            this_iteration = pending
-            pending = set()
-            for version, parents in self.iter_parents(this_iteration):
-                result[version] = parents
-                for parent in parents:
-                    if parent in result:
-                        continue
-                    pending.add(parent)
+            result = self.get_parent_map(self.versions())
+        else:
+            result = {}
+            pending = set(version_ids)
+            while pending:
+                this_iteration = pending
+                pending = set()
+                parents = self.get_parent_map(this_iteration)
+                for version, parents in parents.iteritems():
+                    result[version] = parents
+                    for parent in parents:
+                        if parent in result:
+                            continue
+                        pending.add(parent)
+        references = set()
+        for parents in result.itervalues():
+            references.update(parents)
+        existing_parents = self.get_parent_map(references)
+        for key, parents in result.iteritems():
+            present_parents = [parent for parent in parents if parent in
+                existing_parents]
+            result[key] = tuple(present_parents)
         return result
 
     @deprecated_method(one_four)
@@ -425,6 +422,7 @@
         except KeyError:
             raise errors.RevisionNotPresent(version_id, self)
 
+    @deprecated_method(one_four)
     def annotate_iter(self, version_id):
         """Yield list of (version-id, line) pairs for the specified
         version.
@@ -432,10 +430,15 @@
         Must raise RevisionNotPresent if the given version is
         not present in file history.
         """
-        raise NotImplementedError(self.annotate_iter)
+        return iter(self.annotate(version_id))
 
     def annotate(self, version_id):
-        return list(self.annotate_iter(version_id))
+        """Return a list of (version-id, line) tuples for version_id.
+
+        :raise RevisionNotPresent: If the given version is
+        not present in file history.
+        """
+        raise NotImplementedError(self.annotate)
 
     def join(self, other, pb=None, msg=None, version_ids=None,
              ignore_missing=False):
@@ -477,6 +480,7 @@
         """
         raise NotImplementedError(self.iter_lines_added_or_present_in_versions)
 
+    @deprecated_method(one_four)
     def iter_parents(self, version_ids):
         """Iterate through the parents for many version ids.
 
@@ -488,14 +492,6 @@
         """
         return self.get_parent_map(version_ids).iteritems()
 
-    def transaction_finished(self):
-        """The transaction that this file was opened in has finished.
-
-        This records self.finished = True and should cause all mutating
-        operations to error.
-        """
-        self.finished = True
-
     def plan_merge(self, ver_a, ver_b):
         """Return pseudo-annotation indicating how the two versions merge.
 

=== modified file 'bzrlib/weave.py'
--- bzrlib/weave.py	2008-04-01 02:41:25 +0000
+++ bzrlib/weave.py	2008-04-09 00:34:54 +0000
@@ -185,7 +185,13 @@
     __slots__ = ['_weave', '_parents', '_sha1s', '_names', '_name_map',
                  '_weave_name', '_matcher']
     
-    def __init__(self, weave_name=None, access_mode='w', matcher=None):
+    def __init__(self, weave_name=None, access_mode='w', matcher=None, get_scope=None):
+        """Create a weave.
+
+        :param get_scope: A callable that returns an opaque object to be used
+            for detecting when this weave goes out of scope (should stop
+            answering requests or allowing mutation).
+        """
         super(Weave, self).__init__(access_mode)
         self._weave = []
         self._parents = []
@@ -197,10 +203,22 @@
             self._matcher = bzrlib.patiencediff.PatienceSequenceMatcher
         else:
             self._matcher = matcher
+        if get_scope is None:
+            get_scope = lambda:None
+        self._get_scope = get_scope
+        self._scope = get_scope()
+        self._access_mode = access_mode
 
     def __repr__(self):
         return "Weave(%r)" % self._weave_name
 
+    def _check_write_ok(self):
+        """Is the versioned file marked as 'finished' ? Raise if it is."""
+        if self._get_scope() != self._scope:
+            raise errors.OutSideTransaction()
+        if self._access_mode != 'w':
+            raise errors.ReadOnlyObjectDirtiedError(self)
+
     def copy(self):
         """Return a deep copy of self.
         
@@ -401,11 +419,6 @@
                 offset += 2 + (j2 - j1)
         return new_version
 
-    def _clone_text(self, new_version_id, old_version_id, parents):
-        """See VersionedFile.clone_text."""
-        old_lines = self.get_text(old_version_id)
-        self.add_lines(new_version_id, parents, old_lines)
-
     def _inclusions(self, versions):
         """Return set of all ancestors of given version(s)."""
         if not len(versions):
@@ -453,13 +466,13 @@
         """
         return len(other_parents.difference(my_parents)) == 0
 
-    def annotate_iter(self, version_id):
-        """Yield list of (version-id, line) pairs for the specified version.
+    def annotate(self, version_id):
+        """Return a list of (version-id, line) tuples for version_id.
 
         The index indicates when the line originated in the weave."""
         incls = [self._lookup(version_id)]
-        for origin, lineno, text in self._extract(incls):
-            yield self._idx_to_name(origin), text
+        return [(self._idx_to_name(origin), text) for origin, lineno, text in
+            self._extract(incls)]
 
     def iter_lines_added_or_present_in_versions(self, version_ids=None,
                                                 pb=None):
@@ -670,10 +683,6 @@
                        expected_sha1, measured_sha1))
         return result
 
-    def get_sha1(self, version_id):
-        """See VersionedFile.get_sha1()."""
-        return self._sha1s[self._lookup(version_id)]
-
     def get_sha1s(self, version_ids):
         """See VersionedFile.get_sha1s()."""
         return [self._sha1s[self._lookup(v)] for v in version_ids]
@@ -865,12 +874,12 @@
 
     WEAVE_SUFFIX = '.weave'
     
-    def __init__(self, name, transport, filemode=None, create=False, access_mode='w'):
+    def __init__(self, name, transport, filemode=None, create=False, access_mode='w', get_scope=None):
         """Create a WeaveFile.
         
         :param create: If not True, only open an existing knit.
         """
-        super(WeaveFile, self).__init__(name, access_mode)
+        super(WeaveFile, self).__init__(name, access_mode, get_scope=get_scope)
         self._transport = transport
         self._filemode = filemode
         try:
@@ -891,11 +900,6 @@
         self._save()
         return result
 
-    def _clone_text(self, new_version_id, old_version_id, parents):
-        """See VersionedFile.clone_text."""
-        super(WeaveFile, self)._clone_text(new_version_id, old_version_id, parents)
-        self._save
-
     def copy_to(self, name, transport):
         """See VersionedFile.copy_to()."""
         # as we are all in memory always, just serialise to the new place.

=== modified file 'bzrlib/workingtree_4.py'
--- bzrlib/workingtree_4.py	2008-03-17 21:58:47 +0000
+++ bzrlib/workingtree_4.py	2008-04-09 00:34:54 +0000
@@ -1402,7 +1402,7 @@
                       default_revision=_mod_revision.CURRENT_REVISION):
         """See Tree.annotate_iter"""
         w = self._get_weave(file_id)
-        return w.annotate_iter(self.inventory[file_id].revision)
+        return w.annotate(self.inventory[file_id].revision)
 
     def _get_ancestors(self, default_revision):
         return set(self._repository.get_ancestry(self._revision_id,

